<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------


if (!defined('AUTHORDBMAPPERCLASSPHP'))
{
    define ('AUTHORDBMAPPERCLASSPHP', 'AUTHORDBMAPPERCLASSPHP');

    include ('globals.php');
    include ('exceptionhandling.php');
    include ('loggclass.php');
    include ('authorclass.php');
    include ('basedbclass.php');

    class AuthorDbMapperClass extends BaseDbClass
    {
	public function __construct($dbname, $dbuser, $dbpassword) 
	{
	    parent::__construct($dbname, $dbuser, $dbpassword);
	}
	
	public function __destruct() 
	{
	    LoggClass::logg("destruct authordbmapperclass");
	}
	

	// readAuthors
	// Methode zum Lesen einer vollstaendigen Autorenliste
	// aus der Datenbank.
	public function readAuthors()
	{
	    $returnlist = array();
	    $sql = "select * from t_authors a;";

	    try
		{
		    if ($result = $this->getConnection()->Execute($sql))
			{
			    while (! $result->EOF)
				{
				    $returnlist[$result->fields['authorid']] 
					= new AuthorClass
					    ($result->fields['lastname'], 
					     $result->fields['firstname']);
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen der Autoren");
			    throw new Exception
				("Fehler beim Lesen der Autoren");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr
			("Fehler beim Listen der Autoren");
		    throw new Exception
			("Fehler beim Lesen der Autoren");
		}

	    return $returnlist;
	}


	// readAuthorNames
	// Methode, die eine Liste saemtlicher Autorennamen aus der Datebank
	// zurueckgibt. 
	// Wird benoetigt, um die Autoren-Select-Tags zusammenzusetzen. 
	public function readAuthorNames()
	{
	    $returnlist = array();
	    $sql = "select * from t_authors a;";

	    try
		{
		    if ($result = $this->getConnection()->Execute($sql))
			{
			    while (!$result->EOF)
				{
				    $returnlist[$result->fields['authorid']] 
					= $result->fields['lastname'] . ', '  
					. $result->fields['firstname'];
				    $result->MoveNext();
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen der Autorennamen");
			    // Keine Exception werden, da Methode
			    // innerhalb des HTML verwendet werden soll
			    return array();
			};
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr
			("Fehler beim Lesen der Autorennamen");
		    // Keine Exception werden, da Methode
		    // innerhalb des HTML verwendet werden soll
		    return array();
		}
	    return $returnlist;
	}


	// readAuthor
	// liest einen einzelnen Autor aufgrund der uebergebenen ID aus 
	// der Datenbank. 
	// Parameter: $authorid - ID des auszulesenden Autors. 
	public function readAuthor($authorid)
	{
	    $returnvalue = null;
	    
	    $sql = "select * from t_authors a "
		. "where a.authorid = $authorid ";
	    
	    try
		{
		    if ($result = $this->getConnection()->Execute($sql))
			{
			    if (1 == $result->RecordCount())
				{
				    $returnvalue =  
					new AuthorClass
					    ($result->fields['lastname'], 
					     $result->fields['firstname'] );
				}
			    else // Kein Autor mit der richtigen ID gefunden
				{
				    $returnvalue = new AuthorClass;
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen des Autors");
			    throw new Exception
				("Fehler beim Lesen des Autors");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr 
			("Fehler beim Lesen des Autors");
		    throw new Exception
			("Fehler beim Lesen des Autors");
		}   

	    return $returnvalue;
	}

    }
}
?>
