<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------

if (! defined('BOOKCLASSPHP') )
{
    define ('BOOKCLASSPHP', 'BOOKCLASSPHP' );

    include ('globals.php');
    include ('exceptionhandling.php');
    include ('loggclass.php');
    include ('authorclass.php');    

    // Buch-Klasse zur Darstellung eines Buches
    // mit Autor, AutorID, Titel und Untertitel. 
    class BookClass
    {
	// Das Buchobjekt speichert die DatenbankID des 
	// eigenen Autors ... 
	private $authorid = 0;
	// ... und das Autorobjekt selber. 
	private $author = null;
	private $title = '';
	private $subtitle = '';

	public function __construct($authorid = 0, 
				    $author   = null,  
				    $title    = '', 
				    $subtitle = '')
	{
	    $this->authorid = $authorid;
	    if (isset ($author))
		{
		    // Pruefe, ob ein gueltiges Autor-Objekt 
		    // uebergeben wurde.
		    if ($author instanceof AuthorClass)
			{$this->author = $author;}
		    else 
			{
			    LoggClass::loggErr
				("BookClass::__construct: Ungueltiger Autor");
			    throw new Exception ("Ungueltiger Autor");
			}   
		}
	    else
		{
		    $this->author = new AuthorClass;
		}

	    $this->title    = $title;  
	    $this->subtitle = $subtitle; 
	}
	
	public function __destruct () 	
	{
	    LoggClass::logg("destruct book instance");
	}

	public function setAuthorId ($authorid)
	{ $this->authorid = $authorid; }

	public function getAuthorId () 
	{ return $this->authorid; }

	public function checkAuthorId()
	{ return $this->authorid > 0; }

	public function setAuthor($author)
	{ 
	    if ($author instanceof AuthorClass)
		{$this->author = $author;}
	    else 
		{
		    LoggClass::loggErr
			("BookClass::setAuthor: Ungueltiger Autor");
		    throw new Exception("Ungueltiger Autor");
		}
	}

	public function getAuthor ()
	{ return $this->author; }

	public function getAuthorLastName()
	{ return $this->author->getLastName(); }

	public function getAuthorFirstName()
	{ return $this->author->getFirstName(); }

	public function checkAuthor()
	{return $this->author->checkLastname()
	     && $this->author->checkFirstname();}

	public function setTitle ($title)
	{ $this->title = $title; }
	
	public function getTitle ()
	{ return $this->title; }

	public function checkTitle ()
	{ return ($this->title ? true: false); }

	public function setSubtitle ($subtitle)
	{ $this->subtitle = $subtitle; }

	public function getSubtitle ()
	{ return $this->subtitle; }

	public function checkSubtitle ()
	{ return true; }

	public function isValid()
	{ 
	    return $this->checkAuthorId()
		&& $this->author->isValid() 
		&& $this->checkTitle()
		&& $this->checkSubtitle();
	}
	
	public function isEmpty()
	{
	    return ( ($this->authorid == 0)
		     && $this->author->isEmpty()
		     && ! $this->title 
		     && ! $this->subtitle );
	}

	public function __toString()
	{
	    return "authorid=" . $this->authorid
		. ";author=(" . $this->author->toString()
		. ");title=" . $this->title
		. ";subtitle=" . $this->subtitle;
	}
    }
}
?>
