<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------


if (!defined('BOOKDBMAPPERCLASSPHP'))
{
    define ('BOOKDBMAPPERCLASSPHP', 'BOOKDBMAPPERCLASSPHP');

    include ('globals.php');
    include ('exceptionhandling.php');
    include ('loggclass.php');
    include ('bookclass.php');
    include ('basedbclass.php');

    class BookDbMapperClass extends BaseDbClass
    {
	public function __construct($dbname, $dbuser, $dbpassword) 
	{
	    parent::__construct($dbname, $dbuser, $dbpassword);
	}
	
	public function __destruct() 
	{
	    LoggClass::logg("destruct BookDbMapper Instance");
	}
	

	
	// readBooks
	// Methode zum Auslesen einer vollstaendigen Buecherliste. 
	public function readBooks()
	{
	    // sql Statement um Buecher aus der Datenbank zu lesen
	    $sql = "select * from t_books b, t_authors a "
		. "where a.authorid = b.authorid;";

	    // Initialisiere Buecherliste als leeres Array
	    $returnlist = array();

	    // Lies Buecher aus der Datenbank
	    try{
		if ($result = $this->getConnection()->Execute($sql))
		    {
			// Schleife ueber die Ergebnisliste
			while (!$result->EOF)
			    {
				$returnlist[$result->fields['bookid']] 
				    = new BookClass
					($result->fields['authorid'], 
					 new AuthorClass
					     ($result->fields['lastname'], 
					      $result->fields['firstname']), 
					 $result->fields['title'], 
					 $result->fields['subtitle']);
				$result->MoveNext();
			    }
		    }
		else // ein Fehler ist aufgetreten
		    {
			LoggClass::loggErr  ("Fehler beim Lesen der Buecher");
			throw new Exception ("Fehler beim Lesen der Buecher");
		    };
	    }
	    catch (Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Lesen der Buecher");
		    throw new Exception ("Fehler beim Lesen der Buecher");
		}
	    return $returnlist;
	}

	// readBook
	// Methode zum Lesen eines einzelnen Buches aus der Datenbank.
	// Parameter: $bookid - ID des auszulesenden Buches.
	public function readBook($bookid)
	{
	    $sql = "select * from t_books b, t_authors a "
		. "where a.authorid = b.authorid "
		. "and bookid = $bookid";

	    $returnvalue = null;
	    
	    try
		{
		    if ($result = $this->getConnection()->Execute ($sql))
			{
			    // Buch mit passender ID gefunden?
			    if (1 == $result->RecordCount())
				{
				    $returnvalue = new BookClass 
					($result->fields['authorid'], 
					 new AuthorClass
					     ($result->fields['lastname'], 
					      $result->fields['firstname']), 
					 $result->fields['title'], 
					 $result->fields['subtitle']);
				}
			    else // Kein Buch mit passender ID gefunden
				{
				    $returnvalue = new BookClass;
				}
		    
			}
		    else // Ein Fehler ist aufgetreten
			{
			    LoggClass::loggErr  ("Fehler beim Lesen des Buches");
			    throw new Exception ("Fehler beim Lesen des Buches");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Lesen des Buches");
		    throw new Exception ("Fehler beim Lesen des Buches");
		}

	    return $returnvalue;
	}

	// insertBook
	// fuegt ein neues Buch in die Datenbank ein 
	// und gibt die ID des eingefuegte Buches zurueck.
	// Parameter: $Book - Buchobjekt des einzufuegenden Buches. 
	public function insertBook(& $Book)
	{
	    $sql = "select nextval('s_books')";
	    $returnvalue = null;

	    // Bestimme ID des Buches aus der Sequence s_books
	    try
		{
		    if ($result = $this->getConnection()->Execute($sql)
			and 1 == $result->RecordCount() )
			{
			    $returnvalue = $result->fields['nextval'];
			}
		    else
			{
			    LoggClass::loggErr  ("Fehler beim Bestimmen der Id");
			    throw new Exception ("Fehler beim Bestimmen der Id");
			};
		}    
	    catch(Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Bestimmen der Id");
		    throw new Exception ("Fehler beim Bestimmen der Id");
		}

	    // Lege Buch in Datenbank an
	    $sql = "insert into t_books values "
		. "($returnvalue, " . $Book->getAuthorId()
		. ", '" . $Book->getTitle() . "', "
		. "'" . $Book->getSubtitle() . "')";
		
	    try
		{
		    if (! ($result = $this->getConnection()->Execute($sql)) )
			{
			    LoggClass::loggErr  
				("Fehler beim Speichern des Buches");
			    throw new Exception	
				("Fehler beim Speichern des Buches");
			};
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  
			("Fehler beim Speichern des Buches");
		    throw new Exception	
			("Fehler beim Speichern des Buches");
		}

	    return $returnvalue;	    
	}

	// updateBook
	// aendert ein bestehendes Buch in der Datenbank 
	// aufgrund der uebergebenen Werte. 
	// Parameter: 
	// $bookid - ID des zu aendernden Buches. 
	// $Book - Buchobjekt, das die zu speichernden Werte enthaelt. 
	public function updateBook($bookid, & $Book)
	{
	    $sql = "update t_books set "
		. "authorid = " . $Book->getAuthorId()
		. ", title = '" . $Book->getTitle() . "'"
		. ", subtitle = '" . $Book->getSubtitle() . "'"
		. " where bookid = $bookid";
		
	    try
		{
		    if (! $result = $this->getConnection()->Execute($sql))
			{
			    LoggClass::loggErr  
				("Fehler beim Speichern des Buches");
			    throw new Exception 
				("Fehler beim Speichern des Buches"); 
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  
			("Fehler beim Speichern des Buches");
		    throw new Exception 
			("Fehler beim Speichern des Buches"); 
		}
	    return;	    
	}

	// deleteBook
	// Methode zum Loeschen eines bestehenden Buches in der Datenbank. 
	// Parameter: $bookid - ID des zu loeschenden Buches. 
	public function deleteBook($bookid)
	{
	    $sql = "delete from t_books"
		. " where bookid = $bookid";
		
	    try
		{
		    if (! ($result = $this->getConnection()->Execute($sql)) )
			{
			    LoggClass::loggErr  
				("Fehler beim Loeschen des Buches");
			    throw new Exception 
				("Fehler beim Loeschen des Buches");
			};
		}
	    catch (Exception $e)
			{
			    LoggClass::loggErr  
				("Fehler beim Loeschen des Buches");
			    throw new Exception 
				("Fehler beim Loeschen des Buches");
			};

	    return;	    
	}

    }
}
?>
