<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------


if (!defined('SIMPLEDBMAPPERCLASSPHP'))
{
    define ('SIMPLEDBMAPPERCLASSPHP', 'SIMPLEDBMAPPERCLASSPHP');
    
    include ('globals.php');
    include ('exceptionhandling.php');
    include ('loggclass.php');
    include ('authorclass.php');
    include ('bookclass.php');
    include ('userclass.php');
    include ('adodb/adodb-exceptions.inc.php');
    include('adodb/adodb.inc.php');

    // Datenbankzugriffsklasse
    // die den Zugriff auf eine PostgreSQL-Datenbank 
    // kapselt. 
    // Zum Zugriff auf andere Datenbanksysteme muessen die 
    // einzelnen Zugriffsmethoden geeignet angepasst werden. 

    class SimpleDbMapperClass
    {
	// Statische Parameter fuer Datenbankzugriff
	private static $dbname = false;
	private static $dbuser = false;
	private static $dbpassword = false;

	// Statische Parameter fuer Verbindungsverwaltung
	private static $connection = false;
	private static $instancecounter = 0;

	// Konstruktor
	// Im Konstruktor wird bei Bedarf die Datenbankverbindung
	// hergestellt. 
	public function __construct($dbname, $dbuser, $dbpassword) 
	{
	    if (self::$connection)
		{
		    // Pruefe, ob neue Verbindung mit
		    // bestehender uebereinstimmt
		    if ($dbname != self::$dbname
			or $dbuser != self::$dbuser
			or $dbpassword != self::$dbpassword )
			{
			    LoggClass::loggErr
				("Datenbankparameter sind nicht korrekt");
			    throw new Exception
				("Datenbankparameter sind nicht korrekt");
			}
		}
	    else // Noch keine Verbindung vorhanden
		{
		    self::$dbname     = $dbname;
		    self::$dbuser     = $dbuser;
		    self::$dbpassword = $dbpassword;

		    try
			{
			    self::$connection 
				=  NewADOConnection(DBDRIVER);
			    self::$connection->Connect
				("dbname=" . $dbname 
				 . " user=" . $dbuser 
				 . " password=" . $dbpassword);
			    // force indexing auf recordset fields by name
			    self::$connection->SetFetchMode(ADODB_FETCH_ASSOC);
			}
		    catch (Exception $e)
			{ 
			    LoggClass::logg
				("Fehler beim DB-Verbindungsaufbau");
			    throw new Exception
				("Fehler beim DB-Verbindungsaufbau");
			}

		}

	    // Inkrementiere Instanzenzaehler
	    self::$instancecounter++; 
	}
	
	// Destruktor
	// Im Destruktor der letzten Instanz wird die Datenbankverbindung
	// geschlossen. 
	public function __destruct()
	{
	    LoggClass::logg("destruct SimpleMapper Instance");
	
	    // Dekrementiere Instanzenzaehler
	    self::$instancecounter--;
 
	    // Trenne Datenbankverbindung, 
	    // falls letzte Instanz
	    if (self::$instancecounter <= 0)
		{
		 try
			{
			    self::$connection->Close();
			}
		    catch (Exception $e)
			{
			    // hier keine Ausnahme werfen, da Destruktor
			}   
		}

	}
	
	// readBooks
	// Methode zum Auslesen einer vollstaendigen Buecherliste. 
	public function readBooks()
	{
	    // sql Statement um Buecher aus der Datenbank zu lesen
	    $sql = "select * from t_books b, t_authors a "
		. "where a.authorid = b.authorid;";

	    // Initialisiere Buecherliste als leeres Array
	    $returnlist = array();

	    // Lies Buecher aus der Datenbank
	    try{
		if ($result = self::$connection->Execute($sql))
		    {
			// Schleife ueber die Ergebnisliste
			while (!$result->EOF)
			    {
				$returnlist[$result->fields['bookid']] 
				    = new BookClass
					($result->fields['authorid'], 
					 new AuthorClass
					     ($result->fields['lastname'], 
					      $result->fields['firstname']), 
					 $result->fields['title'], 
					 $result->fields['subtitle']);
				$result->MoveNext();
			    }
		    }
		else // ein Fehler ist aufgetreten
		    {
			LoggClass::loggErr  ("Fehler beim Lesen der Buecher");
			throw new Exception ("Fehler beim Lesen der Buecher");
		    };
	    }
	    catch (Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Lesen der Buecher");
		    throw new Exception ("Fehler beim Lesen der Buecher");
		}
	    return $returnlist;
	}

	// readBook
	// Methode zum Lesen eines einzelnen Buches aus der Datenbank.
	// Parameter: $bookid - ID des auszulesenden Buches.
	public function readBook($bookid)
	{
	    $sql = "select * from t_books b, t_authors a "
		. "where a.authorid = b.authorid "
		. "and bookid = $bookid";

	    $returnvalue = null;
	    
	    try
		{
		    if ($result = self::$connection->Execute ($sql))
			{
			    // Buch mit passender ID gefunden?
			    if (1 == $result->RecordCount())
				{
				    $returnvalue = new BookClass 
					($result->fields['authorid'], 
					 new AuthorClass
					     ($result->fields['lastname'], 
					      $result->fields['firstname']), 
					 $result->fields['title'], 
					 $result->fields['subtitle']);
				}
			    else // Kein Buch mit passender ID gefunden
				{
				    $returnvalue = new BookClass;
				}
		    
			}
		    else // Ein Fehler ist aufgetreten
			{
			    LoggClass::loggErr  ("Fehler beim Lesen des Buches");
			    throw new Exception ("Fehler beim Lesen des Buches");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Lesen des Buches");
		    throw new Exception ("Fehler beim Lesen des Buches");
		}

	    return $returnvalue;
	}

	// insertBook
	// fuegt ein neues Buch in die Datenbank ein 
	// und gibt die ID des eingefuegte Buches zurueck.
	// Parameter: $Book - Buchobjekt des einzufuegenden Buches. 
	public function insertBook(& $Book)
	{
	    $sql = "select nextval('s_books')";
	    $returnvalue = null;

	    // Bestimme ID des Buches aus der Sequence s_books
	    try
		{
		    if ($result = self::$connection->Execute($sql)
			and 1 == $result->RecordCount() )
			{
			    $returnvalue = $result->fields['nextval'];
			}
		    else
			{
			    LoggClass::loggErr  ("Fehler beim Bestimmen der Id");
			    throw new Exception ("Fehler beim Bestimmen der Id");
			};
		}    
	    catch(Exception $e)
		{
		    LoggClass::loggErr  ("Fehler beim Bestimmen der Id");
		    throw new Exception ("Fehler beim Bestimmen der Id");
		}

	    // Lege Buch in Datenbank an
	    $sql = "insert into t_books values "
		. "($returnvalue, " . $Book->getAuthorId()
		. ", '" . $Book->getTitle() . "', "
		. "'" . $Book->getSubtitle() . "')";
		
	    try
		{
		    if (! ($result = self::$connection->Execute($sql)) )
			{
			    LoggClass::loggErr  
				("Fehler beim Speichern des Buches");
			    throw new Exception	
				("Fehler beim Speichern des Buches");
			};
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  
			("Fehler beim Speichern des Buches");
		    throw new Exception	
			("Fehler beim Speichern des Buches");
		}

	    return $returnvalue;	    
	}

	// updateBook
	// aendert ein bestehendes Buch in der Datenbank 
	// aufgrund der uebergebenen Werte. 
	// Parameter: 
	// $bookid - ID des zu aendernden Buches. 
	// $Book - Buchobjekt, das die zu speichernden Werte enthaelt. 
	public function updateBook($bookid, & $Book)
	{
	    $sql = "update t_books set "
		. "authorid = " . $Book->getAuthorId()
		. ", title = '" . $Book->getTitle() . "'"
		. ", subtitle = '" . $Book->getSubtitle() . "'"
		. " where bookid = $bookid";
		
	    try
		{
		    if (! $result = self::$connection->Execute($sql))
			{
			    LoggClass::loggErr  
				("Fehler beim Speichern des Buches");
			    throw new Exception 
				("Fehler beim Speichern des Buches"); 
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr  
			("Fehler beim Speichern des Buches");
		    throw new Exception 
			("Fehler beim Speichern des Buches"); 
		}
	    return;	    
	}

	// deleteBook
	// Methode zum Loeschen eines bestehenden Buches in der Datenbank. 
	// Parameter: $bookid - ID des zu loeschenden Buches. 
	public function deleteBook($bookid)
	{
	    $sql = "delete from t_books"
		. " where bookid = $bookid";
		
	    try
		{
		    if (! ($result = self::$connection->Execute($sql)) )
			{
			    LoggClass::loggErr  
				("Fehler beim Loeschen des Buches");
			    throw new Exception 
				("Fehler beim Loeschen des Buches");
			};
		}
	    catch (Exception $e)
			{
			    LoggClass::loggErr  
				("Fehler beim Loeschen des Buches");
			    throw new Exception 
				("Fehler beim Loeschen des Buches");
			};

	    return;	    
	}

	// readAuthors
	// Methode zum Lesen einer vollstaendigen Autorenliste
	// aus der Datenbank.
	public function readAuthors()
	{
	    $returnlist = array();
	    $sql = "select * from t_authors a;";

	    try
		{
		    if ($result = self::$connection->Execute($sql))
			{
			    while (! $result->EOF)
				{
				    $returnlist[$result->fields['authorid']] 
					= new AuthorClass
					    ($result->fields['lastname'], 
					     $result->fields['firstname']);
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen der Autoren");
			    throw new Exception
				("Fehler beim Lesen der Autoren");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr
			("Fehler beim Listen der Autoren");
		    throw new Exception
			("Fehler beim Lesen der Autoren");
		}

	    return $returnlist;
	}

	// readAuthorNames
	// Methode, die eine Liste saemtlicher Autorennamen aus der Datebank
	// zurueckgibt. 
	// Wird benoetigt, um die Autoren-Select-Tags zusammenzusetzen. 
	public function readAuthorNames()
	{
	    $returnlist = array();
	    $sql = "select * from t_authors a;";

	    try
		{
		    if ($result = self::$connection->Execute($sql))
			{
			    while (!$result->EOF)
				{
				    $returnlist[$result->fields['authorid']] 
					= $result->fields['lastname'] . ', '  
					. $result->fields['firstname'];
				    $result->MoveNext();
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen der Autorennamen");
			    // Keine Exception werden, da Methode
			    // innerhalb des HTML verwendet werden soll
			    return array();
			};
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr
			("Fehler beim Lesen der Autorennamen");
		    // Keine Exception werden, da Methode
		    // innerhalb des HTML verwendet werden soll
		    return array();
		}
	    return $returnlist;
	}

	// readAuthor
	// liest einen einzelnen Autor aufgrund der uebergebenen ID aus 
	// der Datenbank. 
	// Parameter: $authorid - ID des auszulesenden Autors. 
	public function readAuthor($authorid)
	{
	    $returnvalue = null;
	    
	    $sql = "select * from t_authors a "
		. "where a.authorid = $authorid ";
	    
	    try
		{
		    if ($result = self::$connection->Execute($sql))
			{
			    if (1 == $result->RecordCount())
				{
				    $returnvalue =  
					new AuthorClass
					    ($result->fields['lastname'], 
					     $result->fields['firstname'] );
				}
			    else // Kein Autor mit der richtigen ID gefunden
				{
				    $returnvalue = new AuthorClass;
				}
			}
		    else
			{
			    LoggClass::loggErr 
				("Fehler beim Lesen des Autors");
			    throw new Exception
				("Fehler beim Lesen des Autors");
			}
		}
	    catch (Exception $e)
		{
		    LoggClass::loggErr 
			("Fehler beim Lesen des Autors");
		    throw new Exception
			("Fehler beim Lesen des Autors");
		}   

	    return $returnvalue;
	}


	// readUser
	// Methode zum Lesen der Benutzerdaten aus der Datenbank.
	// Parameter: $username - Name des zu lesenden Benutzers
	public function readUser($username)
	{
	    $returnvalue = null;
	    
	    $sql = "select * from t_users  "
		. "where username = '" . $username . "'"; 

	    try
		{
		    if ($result = self::$connection->Execute($sql))
			{
			    if (1 == $result->RecordCount())
				{
				    $returnvalue = new UserClass 
					($result->fields['username'],
					 $result->fields['lastname'], 
					 $result->fields['firstname'], 
					 $result->fields['password']);
				}
			    else 
				{
				    throw new Exception
					("Benutzer nicht gefunden");
				    // Error Handling
				}
			
			}
		    else
			{
			    LoggClass::loggErr  
				("Fehler beim Lesen der Benutzerdaten ");
			    throw new Exception 
				("Fehler beim Lesen der Benutzerdaten ");
			};
		}
	    catch (Exception $e) 
		{ 
		    LoggClass::loggErr  
			("Fehler beim Lesen der Benutzerdaten ");
		    throw new Exception 
			("Fehler beim Lesen der Benutzerdaten ");
		}

	    return $returnvalue;
	}	


    }
}
?>
