<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------


if (!defined('BASEDBCLASSPHP'))
{
    define ('BASEDBCLASSPHP', 'BASEDBCLASSPHP');

    include ('globals.php');
    include ('exceptionhandling.php');
    include ('loggclass.php');

    // BaseDbClass ist abstract, da nur die
    // vererbten Klassen instantiiert werden sollen. 
    
    abstract class BaseDbClass
    {
	// statische Parameter fuer Datenbankzugriff
	private static $dbname = false;
	private static $dbuser = false;
	private static $dbpassword = false;

	// statische Parameter fuer Verbindungsverwaltung
	// Datenbankverbindung muss zugreifbar durch
	// die Klassen sein - daher protected.
	// Alle anderen Attribute sind privat
	private static $connection = false;
	private static $instancecounter = 0;

	public function __construct($dbname, $dbuser, $dbpassword) 
	{
	    if (self::$connection)
		{
		    // Pruefe, ob neue Verbindung mit
		    // bestehender uebereinstimmt
		    if (   $dbname     != self::$dbname
			or $dbuser     != self::$dbuser
			or $dbpassword != self::$dbpassword )
			{
			    LoggClass::loggErr
				("Datenbankname ist nicht korrekt");
			    throw new Exception
				("Datenbankname ist nicht korrekt");
			}
		}
	    else
		{
		    self::$dbname     = $dbname;
		    self::$dbuser     = $dbuser;
		    self::$dbpassword = $dbpassword;

		    self::$connection 
			=  pg_connect("dbname=" . $dbname 
				      . " user=" . $dbuser 
				      . " password=" . $dbpassword);

		    if (!self::$connection)
			{ 
			    LoggClass::logg
				("Fehler beim DB-Verbindungsaufbau");
			    throw new Exception
				("Fehler beim DB-Verbindungsaufbau");
			}
		}

	    self::$instancecounter++; 
	}
	
	public function __destruct()
	{
	    self::$instancecounter--; 
	    if (self::$instancecounter <= 0)
		{
		    self::$connection->pg_close();
		    // hier keine Ausnahme werfen, da Destruktor
		}
	}	
	
	// die DB-Verbindung muss ueber eine geschuetzte
	// Methode zurueckgegeben werden, da direkter
	// Zugriff auf geschuetzte statische Variablen
	// nicht moeglich ist. 
	protected function getConnection()
	{
	    return self::$connection;
	}
    }
}
?>
