<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------


if (!defined('LOGGCLASSPHP'))
{
    define  ('LOGGCLASSPHP', 'LOGGCLASSPHP');
    
    include ('globals.php');
    
    // Logging-Klasse 
    // Verwaltet das Filehandle auf die Logdatei und kuemmert
    // sich um das Loggen aller Meldungen.
    // Die Logging-Klasse ist als Singleton implementiert. 
    class LoggClass
    {
	// statische private Instanz
	private static $instance = false;
	private static $instancecounter = 0;
	
	// Filehandle auf Logdatei
	private $logfile = false;
	
	// Konstruktor. 
	// Der Konstruktor ist privat, da er nur von der Logging-Klasse
	// selber aufgerufen wird. 
	private function __construct() 
	{
	    self::$instancecounter++;

	    // Logfile oeffnen	
	    $this->logfile = fopen (LOGFILENAME, 'a');

	    // falls das Logfile nicht geoeffnet werden konnte
	    // --> geordneter Rueckzug
	    if (! $this->logfile ) 
		{ 
		    throw new Exception 
			("Fehler beim Oeffnen des Logfiles $myfilename"); 
		}
	    $this->_logg("Logfile geoeffnet");
	}
	
	// Destruktor
	public function __destruct()
	{
	    // Erst muss geprueft werden, ob das Filehandle ueberhaupt
	    // vorhanden ist, da der Destruktor auch aufgerufen wird, 
	    // wenn der Konstruktor bereits mit einer Ausnahme
	    // abgebrochen wurde. 
	    // Der Destruktor darf keine Ausnahme werfen!!
	    if ($this->logfile)
		{
		    try {$this->_logg ("Logfile wird geschlossen ...");}
		    catch (Exception $e) {}
		    fclose($this->logfile);
		}
	    self::$instancecounter--;
	}
	
	// getInstance()-Methode - konstruiert bei Bedarf eine
	// Instanz und gibt diese zurueck.
	public static function getInstance()
	{
	    // if ( ! LoggClass::$instance instanceof LoggClass )
	    if (self::$instancecounter <= 0)
		{    
		    // falls beim Instantiieren ein Fehler auftritt, 
		    // soll der nach oben weitergereicht werden. 
		    try { LoggClass::$instance = new LoggClass(); }
		    catch (Exception $e)
			{
			    throw $e;
			}
		}
	    return LoggClass::$instance; 
	}

	// log-Methoden der Instanz - abweichend vom ueblichen 
	// Konzept privat, da statt dessen die statischen Methoden
	// genutzt werden sollen
	private function _logg($message)
	{
	    if (! fputs($this->logfile,  
			date('H:i:s') . "\t" . $message . "\n"))
		{ 
		    throw new Exception 
			("error writing to log file");
		}
	}
	
	private function _loggWrn($message)
	{
	    if (! fputs($this->logfile,  
			date('H:i:s') . "\t" . $message . "\n"))
		{ 
		    throw new Exception 
			("error writing to log file"); 
		}
	}
	
	private function _loggErr($message)
	{
	    if (! fputs($this->logfile,  
			date('H:i:s') . "\t" . $message . "\n"))
		{ 
		    throw new Exception 
			("error writing to log file"); 
		}

	}
	
	// Die oeffentlichen Logg-Methoden. Sie rufen die 
	// privaten Logg-Methoden der Singleton-Instanz auf. 
	public static function logg($message)
	{
	    try { LoggClass::getInstance()->_logg($message); }
	    catch (Exception $e) { throw $e; }
	}
	
	public function loggWrn($message)
	{
	    try { LoggClass::getInstance()->_loggWrn($message); }
	    catch (Exception $e) { throw $e; }
	}

	public static function loggErr($message)
	{
	    try 
		{ 
		    LoggClass::getInstance()->_loggErr
			(  $message); 
		}
	    catch (Exception $e) 
		{ throw $e; }
	}	
    }
}
?>
