<?php
// ---------------------------------------------------------------------------
// PHP Baukasten Beispielanwendung
// Autor:  Susanne Pfeiffer / www.iloq.de
// E-Mail: susanne.pfeiffer@iloq.de
// ---------------------------------------------------------------------------

include ('globals.php');
include ('exceptionhandling.php');
include ('loggclass.php');
include ('session2class.php');
include ('bookdbmapperclass.php');
include ('authordbmapperclass.php');
include ('authorclass.php');
include ('bookclass.php');
include ('html2class.php');

// Globale Benutzermeldung
$message = '';

// Instantiiere Session Objekt
// Falls ein Fehler auftritt 
// --> Abbruch und Aufruf Exception Handler
$Session = new SessionClass();

// Pruefe, ob Benutzer eingeloggt ist. 
if ( $Session->getState() >= 1 ) 
{
    // Datenbankschnittstelle fuer Autoren initialisieren
    try
	{
	    $AuthorDbMapper = new AuthorDbMapperClass
		(DBNAME,DBUSER, DBPASSWORD);
	}
    catch (Exception $e)
	{
	    // Exception Handling. 
	    // Abbruch und Aufruf Exception Handler 
	    // oder Fortsetzung ohne Datenbankzugriff
	}

    // Datenbankschnittstelle fuer Buecher initialisieren
    try
	{
	    $DbMapper = new BookDbMapperClass
		(DBNAME, DBUSER, DBPASSWORD);
	}
    catch (Exception $e)
	{
	    // Exception Handling. 
	    // Abbruch und Aufruf Exception Handler 
	    // oder Fortsetzung ohne Datenbankzugriff
	}

    // BuchId initialisieren
    $bookid = 0;

    // setze Buch aufgrund von POST-Parametern 
    if (isset ($_POST['submitBookChange'])) 
	{  
	    $bookid = $_POST ['hiddenBookId'];
	    $Book =  new BookClass
		(( isset($_POST['selectAuthorId']) 
		   ? $_POST['selectAuthorId'] : 0),
		 new AuthorClass(),      // setze erstmal leeren Autor
		 ( isset($_POST['inputTitle']) 
		   ? $_POST['inputTitle'] : ''), 
		 ( isset($_POST['inputSubtitle']) 
		   ? $_POST['inputSubtitle'] : '')
		 );

	    // lies Autor aufgrund geposteter AutorId aus der Datenbank
	    try
		{
		    if ( isset($_POST['selectAuthorId']) )
			{
			    $Book->setAuthor 
                                ($AuthorDbMapper->readAuthor
				 ($_POST['selectAuthorId']));
			}
		}
	    catch (Exception $e)
		{
		    // Exception Handling
		    // Abbruch und Aufruf Exception Handler 
		    // oder Fortsetzung ohne Autor
		}
	}
	else // wenn keine post-Parameter gesetzt, 
	//  lies Buch aus der Datenbank
	{
	    // pruefe, ob BuchId als Get-Parameter uebergeben
	    if (isset($_GET['bookid']))
		{
		    // Setze BuchId
		    $bookid = $_GET['bookid'];
		    try
			{
			    $Book = $DbMapper->readBook($bookid); 
			}
		    catch (Exception $e)
			{
			    // Exception Handling
			    // Abbruch und Aufruf Exception Handler 
			    // oder Fortsetzung ohne Buch
			}
		}
	    else // weder ein get - noch ein post-Parameter
		//      --> konstruiere neues Buch 
		{ 
		    $Book = new BookClass(); 
		}
	} 

	// Speichere Buecherdaten
	if (isset ($_POST['submitBookChange']))
	{
	    if ($bookid > 0) // Buch muss gespeichert werden
		{
		    try
			{
			    $DbMapper->updateBook($bookid, $Book);
			    $message .= "Buch wurde geaendert";
			}
		    catch (Exception $e)
			{
			    LoggClass::loggErr
				("Fehler beim Speichern des Buches");
			    $message .= "Buch konnte nicht gespeichert werden";
			}
		}
	    else // neues Buch muss angelegt werden
		{
		    try
			{
			    $bookid  = $DbMapper->insertBook($Book);
			    $message .= "Buch wurde angelegt";
			}
		    catch (Exception $e)
			{
			    LoggClass::loggErr
				("Fehler beim Speichern des Buches");
			    $message .= "Buch konnte nicht angelegt werden";
			}
		}
	}
} // if ( $Session->getState() >= 1 ) 
?>



<html>
<head>
</head>
<body>
  <table>
    <tr><td></td><td><h1>PHP Baukasten Beispielanwendung</h1></td></tr>
    <tr>
      <td><?php echo HtmlClass::getNaviBar($Session->getState()) ?></td>
      <td>
        <?php if ($Session->getState() >= 1): ?>
          <h2>Buch anlegen / &auml;ndern </h2>
          <?php echo $message; ?>
          <form action="bookchange2.php" method="POST">
            <input type="hidden" name="hiddenBookId" value="<?php echo $bookid ?>">
            <table border='0'>
              <tr>
                <td>Autor: </td>
                <td>
                  <?php echo HtmlClass::getSelectTag
                        ("selectAuthorId", $AuthorDbMapper->readAuthorNames(), 
                         $Book->getAuthorId()); ?>
                </td>
              </tr>
              <tr>
                <td>Titel:</td> 
                <td>
                  <input type="text" name="inputTitle" value="<?php echo $Book->getTitle(); ?>">
                </td>
              </tr>
              <tr>
                <td>Untertitel: </td>
                <td>
                  <input type="text" name="inputSubtitle" 
                    value="<?php echo $Book->getSubtitle(); ?>">
                </td>
              </tr>
            </table>
            <input type="submit" name="submitBookChange" value="Speichern">
          </form>
      <?php else: ?>
          Bitte loggen Sie sich erst ein.
        <?php endif; ?>
      </td>
    </tr>
  </table>
</body>   
</html>   
