-- comment drop commands if tables not yet exist

drop view v_books;
drop view v_authornames;
drop table t_books;
drop table t_authors;
drop table t_users;
drop sequence s_books;
drop sequence s_authors;

-- create sequences for books and authors

create sequence s_books;
create sequence s_authors;

-- create tables t_authors, t_books, t_users

create table t_authors
(
	authorid	int4		primary key 
			default nextval('s_authors'),
	lastname	        varchar(31)     not null, 
	firstname	        varchar(31)     not null
);

create table t_books
(
	bookid	        int4		primary key
			default nextval('s_books'),
	authorid        int4		not null
		        references t_authors(authorid) 
		        on delete cascade,   
	title	        varchar(127)	not null, 
	subtitle        varchar(255)
);

create table t_users
(
	username	varchar(31)	primary key,
	password	varchar(31)	not null, 
	lastname		varchar(31)	not null, 
	firstname		varchar(31)	not null
); 

-- create some useful views

create view v_books as
       select b.bookid, a.authorid, a.lastname, a.firstname, 
       b.title, b.subtitle
       from t_books b, t_authors a
       where b.authorid = a.authorid
       order by bookid;

create view v_authornames as
       select authorid, lastname || ', ' || firstname
       from t_authors;


-- insert test data

insert into t_authors (authorid, lastname, firstname) 
values (nextval('s_authors'), 'Meyers', 'Scott');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Effektiv C++ Programmieren', 
       '50 Wege zur Verbesserung Ihrer Programme und Entwuerfe');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Mehr Effektiv C++ Programmieren', 
       '35 neue Wege zur Verbesserung Ihrer Entwuerfe und Programme');

insert into t_authors (authorid, lastname, firstname) 
values (nextval('s_authors'), 'Schlossnagle', 'George');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Advanced PHP Programming', 'A practical guide');
	
insert into t_authors (authorid, lastname, firstname) 
values (nextval('s_authors'), 'Fowler', 'Martin');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 
       'Patterns of Enterprise Application Architecture', '');
		
insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Refactoring', 
       'Improving the Design of Existing Code');

insert into t_authors (authorid, lastname, firstname) 
values (nextval('s_authors'), 'Larman', 'Craig');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Applying UML and Patterns', 
       'An Introduction to OOA and OOD and the UP');

insert into t_authors (authorid, lastname, firstname) 
values (nextval('s_authors'), 'Evans', 'Eric');

insert into t_books (authorid, title, subtitle)
values (currval('s_authors'), 'Domain-Driven Design', 
       'Tackling Complexity in the Heart of Software');

insert into t_users values
('phpbox', 'phpbox', 'Pfeiffer', 'Susanne');

